export declare const ROOT_FOCUS_KEY = "SN:ROOT";
export declare const FOCUSED_CLASS_NAME = "is-spatial-focused";
export declare const FOCUSED_SELECTOR_NAME = ".is-spatial-focused";
/** アクションに対して複数の keycode を登録可能 */
declare const DEFAULT_KEY_MAP: {
    left: (string | number)[];
    up: (string | number)[];
    right: (string | number)[];
    down: (string | number)[];
    enter: (string | number)[];
    back: (string | number)[];
};
export type DirectionKeys = keyof Omit<typeof DEFAULT_KEY_MAP, "enter" | "back">;
export type Details = Partial<Record<"event", KeyboardEvent>>;
/**
 * 長押し時の情報を提供する
 * @see https://github.com/NoriginMedia/react-spatial-navigation/pull/55
 **/
export type PressedKeys = {
    pressedKeys: Partial<Record<string, number>>;
};
export type FocusableProps = {
    className: string | undefined;
    focusKey: string;
    parentFocusKey: string;
    focused: boolean;
    hasFocusedChild: boolean;
    setFocus: (focusKey?: string, detail?: any) => void;
    stealFocus: (detail?: any) => void;
    navigateByDirection: (dir: DirectionKeys) => void;
    pauseSpatialNavigation: () => void;
    resumeSpatialNavigation: () => void;
    updateAllSpatialLayouts: () => void;
    getCurrentFocusKey: () => string | null;
};
export type PublicComponentProps = {
    className?: string;
    focusKey?: string;
    focusable?: boolean;
    forgetLastFocusedChild?: boolean;
    trackChildren?: boolean;
    blockNavigationOut?: boolean;
    onBackPress?: (ownProps: FocusableProps, pressedKeys: PressedKeys) => void | false;
    onEnterPress?: (ownProps: FocusableProps, pressedKeys: PressedKeys) => void;
    onEnterRelease?: (ownProps: FocusableProps) => void;
    onArrowPress?: (dir: DirectionKeys, ownProps: FocusableProps, pressedKeys: PressedKeys) => void | false;
    onBecameFocused?: (layout: Component["layout"], ownProps: FocusableProps, details: Details) => void;
    onBecameBlurred?: (layout: Component["layout"], ownProps: FocusableProps, details: Details) => void;
    autoRestoreFocus?: boolean;
    autoDelayFocusToChild?: boolean;
    preferredChildFocusKey?: string | null;
};
export type ComponentProps = {
    node: HTMLElement | null;
    focusKey: string;
    parentFocusKey: string;
    parentFocusKeyTable: Record<string, 1>;
    focusable: boolean;
    blockNavigationOut: boolean;
    preferredChildFocusKey: string | null;
    forgetLastFocusedChild: boolean;
    trackChildren: boolean;
    autoRestoreFocus: Boolean;
    autoDelayFocusToChild: boolean;
    getOnBackPress: () => ((pressedKeys: PressedKeys) => void | false) | null;
    getOnEnterPress: () => ((pressedKeys: PressedKeys) => void) | null;
    getOnEnterRelease: () => (() => void) | null;
    getOnArrowPress: () => ((dir: DirectionKeys, pressedKeys: PressedKeys) => void | false) | null;
    getOnBecameFocused: () => ((layout: Component["layout"], details: Details) => void) | null;
    getOnBecameBlurred: () => ((layout: Component["layout"], details: Details) => void) | null;
    onUpdateFocus: (bool: boolean) => void;
    onUpdateHasFocusedChild: (bool: boolean) => void;
};
export type Component = {
    lastFocusedChildKey: null | string;
    layout: {
        /** width */
        width: number;
        /** height */
        height: number;
        /** 画面（body）からの上辺の y 座標 */
        top: number;
        /** 画面（body）からの左辺の x 座標 */
        left: number;
        node: HTMLElement | null;
    };
} & ComponentProps;
declare class SpatialNavigation {
    /** 全ての focus 可能なコンポーネントのストレージ */
    private focusableComponents;
    /** 現在 focus されている要素の foucsKey を保存 */
    private focusKey;
    /** 現在 focus されている要素の全ての親の focusKey を保存 */
    private parentsHavingFocusedChild;
    /** spatial navigation 有効化フラグ */
    private enabled;
    /** spatial navigation の主要イベントをブロックするためのフラグ */
    private paused;
    private throttle;
    private throttleKeypresses;
    private pressedKeys;
    private keyMap;
    /** document.addEventListener 内での this はデフォルトで window を参照するため this の値を固定する */
    private eventHandler;
    constructor();
    init({ throttle: _throttle, throttleKeypresses: throttleKeypresses }?: {
        throttle?: number | undefined;
        throttleKeypresses?: boolean | undefined;
    }): void;
    destroy(): void;
    setKeyMap(keys: Partial<typeof DEFAULT_KEY_MAP>): void;
    /**
     * isIncremental（down, right）な場合、要素の右辺または下辺の座標を返す。
     * そうでない場合、要素の左辺または上辺の座標を返す。
     * @return {number} {@link getDestinationCoordinate} が返す座標と比較するための座標を返す
     */
    private getOriginCoordinate;
    /**
     * isIncremental（down, right）な場合、要素の左辺または上辺の座標を返す。
     * そうでない場合、要素の右辺または下辺の座標を返す。
     * @return {number} {@link getOriginCoordinate} が返す座標と比較するための座標を返す。
     */
    private getDestinationCoordinate;
    /**
     * {@link Component}（Rect）の一辺を線分として見立て、その始端と終点の座標情報（x,y）を返す。
     * 始端は常に線分の左端または上端となり、終端は常に右端または下端となる。
     */
    private getLineSegment;
    private calculateOverlap;
    /** direction に対して垂直な origin と destination の距離（絶対値）を求める */
    private getPrimaryAxisDistance;
    /** origin と destination の分離軸上（separate axis）の gap を求める */
    private getSecondaryAxisDistance;
    private sortSiblingsByPriority;
    private getEventType;
    private keyDownEvent;
    private keyUpEvent;
    /**
     * NOTE: オリジナルとの差分として、bindEventHandlers 関数内で keyDownEvent/KeyUpEvent を定義せず、メンバー関数として分離。
     */
    private bindEventHandlers;
    private unbindEventHandlers;
    navigateByDirection(direction: DirectionKeys, details?: {}): void;
    /**
     * smartNavigate を実行する。
     *
     * NOTE: オリジナルとの差分として、内部処理を削減のために第２引数に direction を渡すように変更。
     **/
    private onKeyEvent;
    /**
     * direction に基づき、兄弟間を移動するか、親階層へ移動します。
     *
     * NOTE: オリジナルとの差分として sortSiblingsByPriority 実行前に条件分岐を追加
     **/
    private smartNavigate;
    private isParticipatingFocusableComponent;
    /**
     * 引数に与えられた focusKey（FocusableComponent）の　tree 上の最下層の focusKey を返します
     */
    private getNextFocusKey;
    addFocusable({ node, focusKey, parentFocusKey, parentFocusKeyTable, focusable, blockNavigationOut, preferredChildFocusKey, forgetLastFocusedChild, trackChildren, autoRestoreFocus, autoDelayFocusToChild, getOnBackPress, getOnEnterPress, getOnEnterRelease, getOnArrowPress, getOnBecameFocused, getOnBecameBlurred, onUpdateFocus, onUpdateHasFocusedChild, }: ComponentProps): void;
    removeFocusable(focusKey: string): void;
    /**
     * this.focusKey を第一引数の newFocusKey へ更新する。
     * 第一引数の newFocusKey に紐づく FocusableComponent を focused な状態に変更して、
     *   前に focus されていた FocusableComponent を非 focusd な状態に更新する
     *
     * NOTE: オリジナルとの差分として、この関数内では saveLastFocusedChildKey を行わない。
     *   後続で実行する updateParentsLastFocusedChild 関数内で実行で事足りるため。
     */
    private setCurrentFocusedKey;
    getCurrentFocusKey(): string | null;
    /**
     * 最後に focus された FocusableComponent の focuskey を parentComponent の lastFocusedChildKey property に保存する。
     * @param component parentComponent
     * @param focusKey 第一引数の lastFocusedChildKey として登録する focusKey
     */
    private saveLastFocusedChildKey;
    /**
     * 第一引数に与えられた focusKey の全ての parentComponent で {@link saveLastFocusedChildKey} を実行する。
     */
    private updateParentsLastFocusedChild;
    /**
     * this.parentsHasFocusedChild を更新する。
     * それに影響を受ける trackChildren が有効な FocusableParentComponent の hasFocusableChild prop を変更する。
     */
    private updateParentsHasFocusedChild;
    pause(): void;
    resume(): void;
    private __setFocus;
    setFocus(focusKey: string, overwriteFocusKey?: string, details?: Details): void;
    private onEnterRelease;
    private onBackPress;
    private onEnterPress;
    private onArrowPress;
    /**
     * {@link Component} の layout 情報を全て再測定して更新する。
     */
    updateAllLayouts(): void;
    private updateLayout;
    updateFocusable(focusKey: string, updateOption: {
        node: HTMLElement | null;
        preferredChildFocusKey: string | null;
        focusable: boolean;
        blockNavigationOut: boolean;
    }): void;
}
export declare const spatialNavigation: SpatialNavigation;
export {};
